@@ -72,32 +72,46 @@ function policy.policies()
             ["build.rpath"]                       = {description = "Enable build rpath.", default = true, type = "boolean"},
             -- Enable C++ modules for C++ building, even if no .mpp is involved in the compilation
             ["build.c++.modules"]                 = {description = "Enable C++ modules for C++ building.", type = "boolean"},
+            -- Enable two phase compilation for C++ modules if supported by the compiler
+            ["build.c++.modules.two_phases"]      = {description = "Enable two phase compilation if supported.", default = true, type = "boolean"},
             -- Enable std module
             ["build.c++.modules.std"]             = {description = "Enable std modules.", default = true, type = "boolean"},
             -- Enable unreferenced and non-public named module culling
             ["build.c++.modules.culling"]         = {description = "Enable unrefereced and non-public named module culling.", default = true, type = "boolean"},
-            -- Try to reuse compiled module bmi file if targets flags permit it
-            ["build.c++.modules.tryreuse"]        = {description = "Try to reuse compiled module if possible.", default = true, type = "boolean"},
-            -- Enable module taking defines acbount for bmi reuse discrimination
+            -- Always reuse compiled module bmi file
+            ["build.c++.modules.reuse"]           = {description = "Reuse compiled module artifacts if possible.", default = true, type = "boolean"},
+            ["build.c++.modules.tryreuse"]        = {description = "Try to reuse compiled module if possible. (deprecated)", default = false, type = "boolean"},
+            -- Disabled flag check when reusing modules
+            ["build.c++.modules.reuse.nocheck"]   = {description = "Disable flag compatibility check when reusing modules.", default = false, type = "boolean"},
+            -- If target will not reuse modules from target deps if defines are different
+            ["build.c++.modules.reuse.strict"]          = {description = "Enable strict defines comparison when trying to reuse module.", default = false, type = "boolean"},
             ["build.c++.modules.tryreuse.discriminate_on_defines"] = {description = "Enable defines module reuse discrimination.", default = false, type = "boolean"},
+            -- Force C++ modules fallback dependency scanner for all compilers
+            ["build.c++.modules.fallbackscanner"]       = {description = "Force fallback module dependency scanner.", default = false, type = "boolean"},
             -- Force C++ modules fallback dependency scanner for clang
-            ["build.c++.clang.fallbackscanner"]   = {description = "Force clang fallback module dependency scanner.", default = false, type = "boolean"},
+            ["build.c++.modules.clang.fallbackscanner"] = {description = "Force clang fallback module dependency scanner.", default = false, type = "boolean"},
+            ["build.c++.clang.fallbackscanner"]         = {description = "Force clang fallback module dependency scanner. (deprecated)", default = false, type = "boolean"},
             -- Force C++ modules fallback dependency scanner for msvc
-            ["build.c++.msvc.fallbackscanner"]    = {description = "Force msvc fallback module dependency scanner.", default = false, type = "boolean"},
-            -- Set the default vs runtime, e.g. MT, MD
-            ["build.c++.msvc.runtime"]            = {description = "Set the default vs runtime.", type = "string", values = {"MT", "MD"}},
+            ["build.c++.modules.msvc.fallbackscanner"]  = {description = "Force msvc fallback module dependency scanner.", default = false, type = "boolean"},
+            ["build.c++.msvc.fallbackscanner"]          = {description = "Force msvc fallback module dependency scanner. (deprecated)", default = false, type = "boolean"},
             -- Force C++ modules fallback dependency scanner for gcc
-            ["build.c++.gcc.fallbackscanner"]     = {description = "Force gcc fallback module dependency scanner.", default = false, type = "boolean"},
+            ["build.c++.modules.gcc.fallbackscanner"]   = {description = "Force gcc fallback module dependency scanner.", default = false, type = "boolean"},
+            ["build.c++.gcc.fallbackscanner"]           = {description = "Force gcc fallback module dependency scanner. (deprecated)", default = false, type = "boolean"},
             -- Force to enable new cxx11 abi in C++ modules for gcc
             -- If in the future, gcc can support it well, we'll turn it on by default
             -- https://github.com/xmake-io/xmake/issues/3855
-            ["build.c++.gcc.modules.cxx11abi"]    = {description = "Force to enable new cxx11 abi in C++ modules for gcc.", type = "boolean"},
+            ["build.c++.modules.gcc.cxx11abi"]    = {description = "Force to enable new cxx11 abi in C++ modules for gcc.", type = "boolean"},
+            ["build.c++.gcc.modules.cxx11abi"]    = {description = "Force to enable new cxx11 abi in C++ modules for gcc. (deprecated)", type = "boolean"},
+            -- Set the default vs runtime, e.g. MT, MD
+            ["build.c++.msvc.runtime"]            = {description = "Set the default vs runtime.", type = "string", values = {"MT", "MD"}},
             -- Enable cuda device link
             ["build.cuda.devlink"]                = {description = "Enable Cuda devlink.", type = "boolean"},
+            -- Enable build jobgraph
+            ["build.jobgraph"]                    = {description = "Enable build jobgraph.", default = true, type = "boolean"},
             -- Enable windows UAC and set level, e.g. invoker, admin, highest
             ["windows.manifest.uac"]              = {description = "Enable windows manifest UAC.", type = "string"},
             -- Enable ui access for windows UAC
-            ["windows.manifest.uac.ui"]           = {description = "Enable windows manifest UAC.", type = "boolean"},
+            ["windows.manifest.uac.ui"]           = {description = "Enable ui access for windows UAC.", type = "boolean"},
             -- Automatically build before running
             ["run.autobuild"]                     = {description = "Automatically build before running.", type = "boolean"},
             -- Enable install rpath
@@ -136,13 +150,24 @@ function policy.policies()
             -- if true, then any updates to library dependencies, such as buildhash changes due to version changes,
             -- will force the installed packages to be recompiled and installed. @see https://github.com/xmake-io/xmake/issues/2719
             ["package.librarydeps.strict_compatibility"] = {description = "Set strict compatibility for package and it's all library dependencies.", type = "boolean"},
+            -- Resolve package dependencies conflict automatically
+            -- @see https://github.com/xmake-io/xmake/issues/5745
+            ["package.resolve_depconflict"]       = {description = "Automatically resolve package dependencies conflict.", default = true, type = "boolean"},
+            -- Synchronize add_requires configuration in toplevel to all package dependencies for this package
+            -- @see https://github.com/xmake-io/xmake/issues/5745
+            ["package.sync_requires_to_deps"]     = {description = "Synchronize requires configuration to all package dependencies.", default = false, type = "boolean"},
             -- Automatically passes dependency configuration for inner xmake package
-            -- https://github.com/xmake-io/xmake/issues/3952
+            -- @see https://github.com/xmake-io/xmake/issues/3952
             ["package.xmake.pass_depconfs"]       = {description = "Automatically passes dependency configuration for inner xmake package", default = true, type = "boolean"},
             -- It will force cmake package use ninja for build
-            ["package.cmake_generator.ninja"]     = {description = "Set cmake package use ninja for build", default = false, type = "boolean"},
+            ["package.cmake_generator.ninja"]     = {description = "Set cmake package use ninja for build", type = "boolean"},
             -- Enable msbuild MultiToolTask
             ["package.msbuild.multi_tool_task"]   = {description = "Enable msbuild MultiToolTask.", default = false, type = "boolean"},
+            -- Merge all static libraries after installing package
+            -- @see https://github.com/xmake-io/xmake/issues/5894
+            ["package.merge_staticlibs"]          = {description = "Merge all static libraries after installing package", type = "boolean"},
+            -- C/C++ build cache
+            ["package.build.ccache"]              = {description = "Enable C/C++ package build cache.", default = false, type = "boolean"},
             -- Stop to test on the first failure
             ["test.stop_on_first_failure"]        = {description = "Stop to test on the first failure", default = false, type = "boolean"},
             -- Return zero as exit code on failure
@@ -153,7 +178,10 @@ function policy.policies()
             --   private: it will disable fetch remote package repositories
             ["network.mode"]                      = {description = "Set the network mode", type = "string"},
             -- Set the compatibility version, e.g. 2.0, 3.0
-            ["compatibility.version"]             = {description = "Set the compatibility version", type = "string", values = {"2.0", "3.0"}}
+            ["compatibility.version"]             = {description = "Set the compatibility version", type = "string", default = "3.0", values = {"2.0", "3.0"}},
+            -- Generate the solution file in root output directory
+            -- @see https://github.com/xmake-io/xmake/issues/6519
+            ["generator.vsxmake.root_sln"]        = {description = "Generate the solution file in root output directory", default = false, type = "boolean"}
         }
         policy._POLICIES = policies
     end
