@@ -32,6 +32,7 @@ local global                = require("base/global")
 local process               = require("base/process")
 local hashset               = require("base/hashset")
 local baseoption            = require("base/option")
+local semver                = require("base/semver")
 local deprecated            = require("base/deprecated")
 local interpreter           = require("base/interpreter")
 local instance_deps         = require("base/private/instance_deps")
@@ -107,12 +108,28 @@ end
 
 -- the current config is belong to the given config values?
 function project._api_is_config(interp, name, ...)
-    return config.is_value(name, ...)
+    local value = config.get(name)
+    local namespace = interp:namespace()
+    if value == nil and namespace then
+        value = config.get(namespace .. "::" .. name)
+    end
+    return config._is_value(value, ...)
 end
 
 -- some configs are enabled?
 function project._api_has_config(interp, ...)
-    return config.has(...)
+    local names = table.pack(...)
+    local namespace = interp:namespace()
+    for _, name in ipairs(names) do
+        local value = config.get(name)
+        if value == nil and namespace then
+            value = config.get(namespace .. "::" .. name)
+        end
+        if value then
+            return true
+        end
+    end
+    return false
 end
 
 -- some packages are enabled?
@@ -120,8 +137,19 @@ function project._api_has_package(interp, ...)
     -- only for loading targets
     local requires = project._memcache():get("requires")
     if requires then
-        for _, name in ipairs(table.pack(...)) do
-            local pkg = requires[name]
+        for _, packagename in ipairs(table.pack(...)) do
+            local pkg = requires[packagename]
+            -- attempt to get package with namespace
+            if pkg == nil and packagename:find("::", 1, true) then
+                local parts = packagename:split("::", {plain = true})
+                local namespace_pkg = requires[parts[#parts]]
+                if namespace_pkg and namespace_pkg:namespace() then
+                    local fullname = namespace_pkg:fullname()
+                    if fullname:endswith(packagename) then
+                        pkg = namespace_pkg
+                    end
+                end
+            end
             if pkg and pkg:enabled() then
                 return true
             end
@@ -131,7 +159,12 @@ end
 
 -- get config from the given name
 function project._api_get_config(interp, name)
-    return config.get(name)
+    local value = config.get(name)
+    local namespace = interp:namespace()
+    if value == nil and namespace then
+        value = config.get(namespace .. "::" .. name)
+    end
+    return value
 end
 
 -- add module directories
@@ -169,6 +202,17 @@ function project._api_add_platformdirs(interp, ...)
     end
 end
 
+-- add toolchain directories
+function project._api_add_toolchaindirs(interp, ...)
+    local scriptdir = project.interpreter():scriptdir()
+    for _, dir in ipairs({...}) do
+        if not path.is_absolute(dir) then
+            dir = path.absolute(dir, scriptdir)
+        end
+        toolchain.add_directories(dir)
+    end
+end
+
 -- load the project file
 function project._load(force, disable_filter)
 
@@ -373,7 +417,8 @@ function project._load_targets()
         for _, sourcefile in ipairs(table.wrap(t:get("files"))) do
             local extension = path.extension((sourcefile:gsub("|.*$", "")))
             if not extensions[extension] then
-                local lang = language.load_ex(extension)
+                local sourcekind = t:extraconf("files", sourcefile, "sourcekind")
+                local lang = sourcekind and language.load_sk(sourcekind) or language.load_ex(extension)
                 if lang and lang:rules() then
                     table.join2(rulenames, lang:rules())
                 end
@@ -382,7 +427,7 @@ function project._load_targets()
         end
         rulenames = table.unique(rulenames)
         for _, rulename in ipairs(rulenames) do
-            local r = project.rule(rulename) or rule.rule(rulename)
+            local r = project.rule(rulename, {namespace = t:namespace()}) or rule.rule(rulename)
             if r then
                 -- only add target rules
                 if r:kind() == "target" then
@@ -500,8 +545,12 @@ function project._load_requires()
     requires_extra = requires_extra or {}
     for _, requirestr in ipairs(table.wrap(requires_str)) do
 
-        -- get the package name
+        -- get the package name, e.g. packagename[foo,bar] >1.0
         local packagename = requirestr:split("%s")[1]
+        local packagename_raw, _ = packagename:match("(.-)%[(.*)%]")
+        if packagename_raw and not packagename:find("::", 1, true) then
+            packagename = packagename_raw
+        end
 
         -- get alias and requireconfs
         local alias = nil
@@ -519,7 +568,7 @@ function project._load_requires()
         end
 
         -- add require info
-        requires[alias or packagename] = instance
+        requires[name] = instance
     end
     return requires
 end
@@ -605,22 +654,23 @@ function project.apis()
     ,   custom =
         {
             -- is_xxx
-            {"is_os",                   project._api_is_os            }
-        ,   {"is_kind",                 project._api_is_kind          }
-        ,   {"is_arch",                 project._api_is_arch          }
-        ,   {"is_mode",                 project._api_is_mode          }
-        ,   {"is_plat",                 project._api_is_plat          }
-        ,   {"is_cross",                project._api_is_cross         }
-        ,   {"is_config",               project._api_is_config        }
+            {"is_os",                   project._api_is_os             }
+        ,   {"is_kind",                 project._api_is_kind           }
+        ,   {"is_arch",                 project._api_is_arch           }
+        ,   {"is_mode",                 project._api_is_mode           }
+        ,   {"is_plat",                 project._api_is_plat           }
+        ,   {"is_cross",                project._api_is_cross          }
+        ,   {"is_config",               project._api_is_config         }
             -- get_xxx
-        ,   {"get_config",              project._api_get_config       }
+        ,   {"get_config",              project._api_get_config        }
             -- has_xxx
-        ,   {"has_config",              project._api_has_config       }
-        ,   {"has_package",             project._api_has_package      }
+        ,   {"has_config",              project._api_has_config        }
+        ,   {"has_package",             project._api_has_package       }
             -- add_xxx
-        ,   {"add_moduledirs",          project._api_add_moduledirs   }
-        ,   {"add_plugindirs",          project._api_add_plugindirs   }
-        ,   {"add_platformdirs",        project._api_add_platformdirs }
+        ,   {"add_moduledirs",          project._api_add_moduledirs    }
+        ,   {"add_plugindirs",          project._api_add_plugindirs    }
+        ,   {"add_platformdirs",        project._api_add_platformdirs  }
+        ,   {"add_toolchaindirs",       project._api_add_toolchaindirs }
         }
     }
 end
@@ -685,9 +735,12 @@ function project.interpreter()
         -- check
         assert(variable)
 
-        -- hack buildir first
-        if variable == "buildir" then
-            return config.buildir()
+        -- hack builddir first
+        if variable == "builddir" or variable == "buildir" then
+            if variable == "buildir" then
+                utils.warning("$(buildir) has been deprecated, please use $(builddir)")
+            end
+            return config.builddir()
         end
 
         -- attempt to get it directly from the configure
@@ -780,6 +833,10 @@ function project.filelock()
 end
 
 -- get the root configuration
+--
+-- get root values in project, e.g project.get("name")
+-- get root values in target, e.g. project.get("target.name")
+-- get root values in specific namespace, e.g. project.get("ns1::ns2::name"), project.get("target.ns1::ns2::name")
 function project.get(name)
     local rootinfo
     if name and name:startswith("target.") then
@@ -819,11 +876,40 @@ function project.version()
     return project.get("target.version")
 end
 
+-- get the project namespaces
+function project.namespaces()
+    return project.interpreter():namespaces()
+end
+
+-- init default policies
+-- @see https://github.com/xmake-io/xmake/issues/5527
+function project._init_default_policies()
+    local compatibility_version = project.policy("compatibility.version")
+    if compatibility_version then
+        if semver.compare(compatibility_version, "3.0") >= 0 then
+            policy.set_default("package.cmake_generator.ninja", true)
+            policy.set_default("build.c++.msvc.runtime", "MD")
+            policy.set_default("run.autobuild", true)
+        else
+            policy.set_default("package.cmake_generator.ninja", false)
+            policy.set_default("build.c++.msvc.runtime", "MT")
+        end
+    end
+end
+
 -- get the project policy, the root policy of the target scope
 function project.policy(name)
     local policies = project._memcache():get("policies")
     if not policies then
 
+        -- init default policies
+        if name ~= "compatibility.version" then
+            if not project._DEFAULT_POLICIES_INITED then
+                project._init_default_policies()
+                project._DEFAULT_POLICIES_INITED = true
+            end
+        end
+
         -- get policies from project, e.g. set_policy("xxx", true)
         policies = project.get("target.policy")
 
@@ -883,9 +969,16 @@ function project.is_loaded()
 end
 
 -- get the given target
-function project.target(name)
+function project.target(name, opt)
+    opt = opt or {}
     local targets = project.targets()
-    return targets and targets[name]
+    if targets then
+        local t = targets[name]
+        if not t and opt.namespace then
+            t = targets[opt.namespace .. "::" .. name]
+        end
+        return t
+    end
 end
 
 -- add the given target, @note if the target name is the same, it will be replaced
@@ -935,8 +1028,16 @@ function project.ordertargets()
 end
 
 -- get the given option
-function project.option(name)
-    return project.options()[name]
+function project.option(name, opt)
+    opt = opt or {}
+    local options = project.options()
+    if options then
+        local o = options[name]
+        if not o and opt.namespace then
+            o = options[opt.namespace .. "::" .. name]
+        end
+        return o
+    end
 end
 
 -- get options
@@ -986,11 +1087,38 @@ function project.requires_str()
 
         -- get raw requires
         requires_str, requires_extra = project.get("requires"), project.get("__extra_requires")
+        local namespaces = project.namespaces()
+        if namespaces then
+            for _, namespace in ipairs(namespaces) do
+                local ns_requires_str, ns_requires_extra = project.get(namespace .. "::requires"), project.get(namespace .. "::__extra_requires")
+                if ns_requires_str then
+                    requires_str = table.wrap(requires_str)
+                    table.join2(requires_str, ns_requires_str)
+                end
+                if ns_requires_extra then
+                    requires_extra = table.wrap(requires_extra)
+                    table.join2(requires_extra, ns_requires_extra)
+                end
+            end
+        end
         project._memcache():set("requires_str", requires_str or false)
         project._memcache():set("requires_extra", requires_extra)
 
         -- get raw requireconfs
         local requireconfs_str, requireconfs_extra = project.get("requireconfs"), project.get("__extra_requireconfs")
+        if namespaces then
+            for _, namespace in ipairs(project.namespaces()) do
+                local ns_requireconfs_str, ns_requireconfs_extra = project.get(namespace .. "::requireconfs"), project.get(namespace .. "::__extra_requireconfs")
+                if ns_requireconfs_str then
+                    requireconfs_str = table.wrap(requireconfs_str)
+                    table.join2(requireconfs_str, ns_requireconfs_str)
+                end
+                if ns_requireconfs_extra then
+                    requireconfs_extra = table.wrap(requireconfs_extra)
+                    table.join2(requireconfs_extra, ns_requireconfs_extra)
+                end
+            end
+        end
         project._memcache():set("requireconfs_str", requireconfs_str or false)
         project._memcache():set("requireconfs_extra", requireconfs_extra)
     end
@@ -1002,6 +1130,42 @@ function project.requireconfs_str()
     project.requires_str()
     local requireconfs_str   = project._memcache():get("requireconfs_str")
     local requireconfs_extra = project._memcache():get("requireconfs_extra")
+    -- synchronize requires configuration to all package dependencies.
+    -- @see https://github.com/xmake-io/xmake/issues/5745#issuecomment-2513951471
+    if project.policy("package.sync_requires_to_deps") then
+        local requires_str   = project._memcache():get("requires_str")
+        local requires_extra = project._memcache():get("requires_extra")
+        local sync_requires_to_deps = project._memcache():get("package.sync_requires_to_deps")
+        if requires_str and not sync_requires_to_deps then
+            requires_extra = requires_extra and table.wrap(requires_extra) or {}
+            requireconfs_str = requireconfs_str and table.wrap(requireconfs_str) or {}
+            requireconfs_extra = requireconfs_extra and table.wrap(requireconfs_extra) or {}
+            for _, require_str in ipairs(table.wrap(requires_str)) do
+                if not require_str:find("::", 1, true) then
+                    local splitinfo = require_str:split("%s")
+                    local packagename = splitinfo[1]
+                    local packageversion = splitinfo[2]
+                    local requireconf_str = "**." .. packagename
+                    local requireconf_extra = table.clone(requires_extra[require_str])
+                    if requireconf_extra then
+                        requireconf_extra.configs = table.clone(requireconf_extra.configs) or {}
+                    end
+                    if packageversion then
+                        requireconf_extra = requireconf_extra or {configs = {}}
+                        requireconf_extra.configs.version = packageversion
+                    end
+                    if requireconf_extra then
+                        requireconf_extra.override = true
+                        table.insert(requireconfs_str, requireconf_str)
+                        requireconfs_extra[requireconf_str] = requireconf_extra
+                    end
+                end
+            end
+            project._memcache():set("requireconfs_str", requireconfs_str)
+            project._memcache():set("requireconfs_extra", requireconfs_extra)
+            project._memcache():set("package.sync_requires_to_deps", true)
+        end
+    end
     return requireconfs_str, requireconfs_extra
 end
 
@@ -1016,8 +1180,13 @@ function project.requireslock_version()
 end
 
 -- get the given rule
-function project.rule(name)
-    return project.rules()[name]
+function project.rule(name, opt)
+    opt = opt or {}
+    local r = project.rules()[name]
+    if r == nil and opt.namespace then
+        r = project.rules()[opt.namespace .. "::" .. name]
+    end
+    return r
 end
 
 -- get project rules
@@ -1036,8 +1205,12 @@ end
 
 -- get the given toolchain
 function project.toolchain(name, opt)
+    opt = opt or {}
     local toolchain_name = toolchain.parsename(name) -- we need to ignore `@packagename`
     local info = project._toolchains()[toolchain_name]
+    if info == nil and opt.namespace then
+        info = project._toolchains()[opt.namespace .. "::" .. toolchain_name]
+    end
     if info then
         return toolchain.load_withinfo(name, info, opt)
     end
@@ -1120,7 +1293,7 @@ function project.menu()
         options_by_category[category] = options_by_category[category] or {}
 
         -- append option to the current category
-        options_by_category[category][opt:name()] = opt
+        options_by_category[category][opt:fullname()] = opt
     end
 
     -- make menu by category
