@@ -42,7 +42,12 @@ local sandbox_module = require("sandbox/modules/import/core/sandbox/module")
 -- new an instance
 function _instance.new(name, info, cachekey, is_builtin, configs)
     local instance       = table.inherit(_instance)
-    instance._NAME       = name
+    local parts = name:split("::", {plain = true})
+    instance._NAME = parts[#parts]
+    table.remove(parts)
+    if #parts > 0 then
+        instance._NAMESPACE = table.concat(parts, "::")
+    end
     instance._INFO       = info
     instance._IS_BUILTIN = is_builtin
     instance._CACHE      = toolchain._localcache()
@@ -68,6 +73,17 @@ function _instance:name()
     return self._NAME
 end
 
+-- get the namespace
+function _instance:namespace()
+    return self._NAMESPACE
+end
+
+-- get the full name
+function _instance:fullname()
+    local namespace = self:namespace()
+    return namespace and namespace .. "::" .. self:name() or self:name()
+end
+
 -- get toolchain platform
 function _instance:plat()
     return self._PLAT or self:config("plat")
@@ -167,7 +183,7 @@ end
 function _instance:is_cross()
     if self:kind() == "cross" then
         return true
-    elseif self:kind() == "standalone" and (self:cross() or self:sdkdir()) then
+    elseif self:kind() == "standalone" and (self:cross() or self:config("sdkdir") or self:info():get("sdkdir")) then
         return true
     end
 end
@@ -239,7 +255,7 @@ end
 -- get the bin directory
 function _instance:bindir()
     local bindir = self:config("bindir") or config.get("bin") or self:info():get("bindir")
-    if not bindir and self:is_cross() and self:sdkdir() and os.isdir(path.join(self:sdkdir(), "bin")) then
+    if not bindir and self:sdkdir() and os.isdir(path.join(self:sdkdir(), "bin")) then
         bindir = path.join(self:sdkdir(), "bin")
     end
     return bindir
@@ -273,23 +289,25 @@ end
 
 -- do check, we only check it once for all architectures
 function _instance:check()
-    local checkok = true
-    local checked = self:_is_checked()
-    if not checked then
+    local checked = self:config("__checked")
+    if checked == nil then
         local on_check = self:_on_check()
         if on_check then
             local ok, results_or_errors = sandbox.load(on_check, self)
             if ok then
-                checkok = results_or_errors
+                checked = results_or_errors
             else
                 os.raise(results_or_errors)
             end
+        else
+            checked = true
         end
         -- we need to persist this state
-        self:config_set("__checked", true)
+        checked = checked or false
+        self:config_set("__checked", checked)
         self:configs_save()
     end
-    return checkok
+    return checked
 end
 
 -- do load manually, it will call on_load()
@@ -386,7 +404,7 @@ end
 
 -- is checked?
 function _instance:_is_checked()
-    return self:config("__checked") == true or self:_on_check() == nil
+    return self:config("__checked") ~= nil or self:_on_check() == nil
 end
 
 -- get the tool description from the tool kind
@@ -436,7 +454,11 @@ function _instance:_description(toolkind)
             nc         = "the nim compiler",
             ncld       = "the nim linker",
             ncsh       = "the nim shared library linker",
-            ncar       = "the nim static library archiver"
+            ncar       = "the nim static library archiver",
+            kc         = "the kotlin native compiler",
+            kcld       = "the kotlin native linker",
+            kcsh       = "the kotlin native shared library linker",
+            kcar       = "the kotlin native static library archiver",
         }
         self._DESCRIPTIONS = descriptions
     end
@@ -499,6 +521,11 @@ function _instance:_checktool(toolkind, toolpath)
     if tool then
         program = tool.program
         toolname = toolname or tool.name
+    else
+        -- we need reset result if not found
+        -- https://github.com/xmake-io/xmake/discussions/6415#discussioncomment-13099816
+        program = nil
+        toolname = nil
     end
 
     -- get tool description from the tool kind
@@ -622,6 +649,15 @@ function toolchain.directories()
     return dirs
 end
 
+-- add toolchain directories
+function toolchain.add_directories(...)
+    local dirs = toolchain.directories()
+    for _, dir in ipairs({...}) do
+        table.insert(dirs, 1, dir)
+    end
+    toolchain._DIRS = table.unique(dirs)
+end
+
 -- load toolchain
 function toolchain.load(name, opt)
 
