@@ -31,7 +31,362 @@ import("core.language.language")
 import("utils.progress")
 import("private.cache.build_cache")
 import("private.service.distcc_build.client", {alias = "distcc_build_client"})
+import("rules.c++.modules.support", {rootdir = os.programdir()})
 
+-- has -static-libstdc++?
+function _has_static_libstdcxx(self)
+    local has_static_libstdcxx = _g._HAS_STATIC_LIBSTDCXX
+    if has_static_libstdcxx == nil then
+        if self:has_flags("-static-libstdc++ -Werror", "ldflags", {flagskey = "gcc_static_libstdcxx"}) then
+            has_static_libstdcxx = true
+        end
+        has_static_libstdcxx = has_static_libstdcxx or false
+        _g._HAS_STATIC_LIBSTDCXX = has_static_libstdcxx
+    end
+    return has_static_libstdcxx
+end
+
+-- get implib file
+function _get_implibfile(self, targetkind, targetfile, opt)
+    if targetkind == "shared" and self:is_plat("mingw") then
+        local target = opt and opt.target
+        local implibfile
+        if target and target:type() == "target" then
+            implibfile = target:artifactfile("implib")
+        end
+        if not implibfile then
+            implibfile = path.join(path.directory(targetfile), path.basename(targetfile) .. ".a")
+        end
+        return implibfile
+    end
+end
+
+-- is cosmocc?
+function _is_cosmocc(self)
+    local is_cosmocc = _g._IS_COSMOCC
+    if is_cosmocc == nil then
+        if self:name():startswith("cosmoc") then
+            is_cosmocc = true
+        end
+        _g._IS_COSMOCC = is_cosmocc or false
+    end
+    return is_cosmocc
+end
+
+-- get `-MMD -MF depfile.d` flags, some old gcc does not support it at same time
+function _get_depfile_flags(self)
+    local depfile_flags = _g._DEPFILE_FLAGS
+    if depfile_flags == nil then
+        local nuldev = os.nuldev()
+        if _is_cosmocc(self) then
+            nuldev = path.unix(os.tmpfile())
+        end
+        if self:has_flags({"-MMD", "-MF", nuldev}, "cxflags", { flagskey = "-MMD -MF" }) then
+            depfile_flags = {"-MMD", "-MF"}
+        end
+        _g._DEPFILE_FLAGS = depfile_flags or false
+    end
+    return depfile_flags
+end
+
+-- get color diagnostics flag
+function _get_color_diagnostics_flag(self)
+    local colors_diagnostics = _g._COLOR_DIAGNOSTICS
+    if colors_diagnostics == nil then
+        if io.isatty() and (tty.has_color8() or tty.has_color256()) then
+            local theme = colors.theme()
+            if theme and theme:name() ~= "plain" then
+                -- for gcc
+                if self:has_flags("-fdiagnostics-color=always", "cxflags") then
+                    colors_diagnostics = "-fdiagnostics-color=always"
+                -- for clang
+                elseif self:has_flags("-fcolor-diagnostics", "cxflags") then
+                    colors_diagnostics = "-fcolor-diagnostics"
+                end
+
+                -- enable color output for windows, @see https://github.com/xmake-io/xmake-vscode/discussions/260
+                if colors_diagnostics and self:name() == "clang" and is_host("windows") and
+                    self:has_flags("-fansi-escape-codes", "cxflags") then
+                    colors_diagnostics = table.join(colors_diagnostics, "-fansi-escape-codes")
+                end
+            end
+        end
+        colors_diagnostics = colors_diagnostics or false
+        _g._COLOR_DIAGNOSTICS = colors_diagnostics
+    end
+    return colors_diagnostics
+end
+
+-- has gnu-line-marker flag?
+function _has_gnu_line_marker_flag(self)
+    local gnu_line_marker = _g._HAS_GNU_LINE_MARKER
+    if gnu_line_marker == nil then
+        if self:has_flags({"-Wno-gnu-line-marker", "-Werror"}, "cxflags") then
+            gnu_line_marker = true
+        end
+        gnu_line_marker = gnu_line_marker or false
+        _g._HAS_GNU_LINE_MARKER = gnu_line_marker
+    end
+    return gnu_line_marker
+end
+
+-- get preprocess file path
+function _get_cppfile(sourcefile, objectfile)
+    return path.join(path.directory(objectfile), "__cpp_" .. path.basename(objectfile) .. path.extension(sourcefile))
+end
+
+-- do preprocess
+function _preprocess(program, argv, opt)
+
+    -- is gcc or clang?
+    local tool = opt.tool
+    local is_gcc = false
+    local is_clang = false
+    if tool then
+        if tool:name() == "gcc" or tool:name() == "gxx" then
+            is_gcc = true
+        elseif tool:name():startswith("clang") then
+            is_clang = true
+        elseif tool:name() == "circle" then
+            return
+        end
+    end
+
+    -- enable "-fdirectives-only"? we need to enable it manually
+    --
+    -- @see https://github.com/xmake-io/xmake/issues/2603
+    -- https://github.com/xmake-io/xmake/issues/2425
+    local directives_only
+    if is_gcc then
+        local cachekey = "core.tools." .. tool:name()
+        directives_only = memcache.get(cachekey, "directives_only")
+        if directives_only == nil then
+            if os.isfile(os.projectfile()) and project.policy("preprocessor.gcc.directives_only") then
+                directives_only = true
+            end
+            memcache.set(cachekey, "directives_only", directives_only)
+        end
+    end
+
+    -- get flags and source file
+    local flags = {}
+    local cppflags = {}
+    local skipped = program:endswith("cache") and 1 or 0
+    for _, flag in ipairs(argv) do
+        if flag == "-o" then
+            break
+        end
+
+        -- get preprocessor flags
+        table.insert(cppflags, flag)
+
+        -- for c++ modules, we cannot support it for clang now
+        if is_clang and flag:startswith("-fmodules") then
+            return
+        end
+
+        -- we cannot enable "-fdirectives-only"
+        if directives_only and (flag:startswith("-D__TIME__=") or
+                flag:startswith("-D__DATE__=") or flag:startswith("-D__TIMESTAMP__=")) then
+            directives_only = false
+        end
+
+        -- get compiler flags
+        if flag == "-MMD" or (flag:startswith("-I") and #flag > 2) or flag:startswith("--sysroot=") then
+            skipped = 1
+        elseif flag == "-MF" or
+            flag == "-I" or flag == "-isystem" or flag == "-include" or flag == "-include-pch" or
+            flag == "-isysroot" or flag == "-gcc-toolchain" then
+            skipped = 2
+        elseif flag:endswith("xcrun") then
+            skipped = 4
+        end
+        if skipped > 0 then
+            skipped = skipped - 1
+        else
+            table.insert(flags, flag)
+        end
+    end
+    local objectfile = argv[#argv - 1]
+    local sourcefile = argv[#argv]
+    assert(objectfile and sourcefile, "%s: iorunv(%s): invalid arguments!", self, program)
+
+    -- is precompiled header?
+    if objectfile:endswith(".gch") or objectfile:endswith(".pch") then
+        return
+    end
+
+    -- disable linemarkers?
+    local linemarkers = _g.linemarkers
+    if linemarkers == nil then
+        if os.isfile(os.projectfile()) and project.policy("preprocessor.linemarkers") == false then
+            linemarkers = false
+        else
+            linemarkers = true
+        end
+        _g.linemarkers = linemarkers
+    end
+
+    -- do preprocess
+    local cppfile = _get_cppfile(sourcefile, objectfile)
+    local cppfiledir = path.directory(cppfile)
+    if not os.isdir(cppfiledir) then
+        os.mkdir(cppfiledir)
+    end
+    table.insert(cppflags, "-E")
+    -- it will be faster for preprocessing
+    -- when preprocessing, handle directives, but do not expand macros.
+    if directives_only then
+        table.insert(cppflags, "-fdirectives-only")
+    end
+    if linemarkers == false then
+        table.insert(cppflags, "-P")
+    end
+    -- if we want to support pch for gcc, we need to enable this flag
+    -- and clang need not this flag, it will use '-include-pch' to include and preprocess header files
+    -- but it will be slower than non-ccache mode.
+    --
+    -- @see https://github.com/xmake-io/xmake/issues/5858
+    -- https://musescore.org/en/node/182331
+    if is_gcc then
+        table.insert(cppflags, "-fpch-preprocess")
+    end
+    table.insert(cppflags, "-o")
+    table.insert(cppflags, cppfile)
+    table.insert(cppflags, sourcefile)
+
+    -- we need to mark as it when compiling the preprocessed source file
+    -- it will indicate to the preprocessor that the input file has already been preprocessed.
+    if is_gcc then
+        table.insert(flags, "-fpreprocessed")
+    end
+    -- with -fpreprocessed, predefinition of command line and most builtin macros is disabled.
+    if directives_only then
+        table.insert(flags, "-fdirectives-only")
+    end
+
+    -- suppress -Wgnu-line-marker warnings
+    -- @see https://github.com/xmake-io/xmake/issues/5737
+    if (is_gcc or is_clang) and _has_gnu_line_marker_flag(tool) then
+        table.insert(flags, "-Wno-gnu-line-marker")
+    end
+
+    -- do preprocess
+    local cppinfo = try {function ()
+        if is_host("windows") then
+            cppflags = winos.cmdargv(cppflags, {escape = true})
+        end
+        local outdata, errdata = os.iorunv(program, cppflags, opt)
+        return {outdata = outdata, errdata = errdata,
+                sourcefile = sourcefile, objectfile = objectfile, cppfile = cppfile, cppflags = flags}
+    end}
+    if not cppinfo then
+        if is_gcc then
+            local cachekey = "core.tools." .. tool:name()
+            memcache.set(cachekey, "directives_only", false)
+        end
+    end
+    return cppinfo
+end
+
+-- compile preprocessed file
+function _compile_preprocessed_file(program, cppinfo, opt)
+    local argv = table.join(cppinfo.cppflags, "-o", cppinfo.objectfile, cppinfo.cppfile)
+    if is_host("windows") then
+        argv = winos.cmdargv(argv, {escape = true})
+    end
+    local outdata, errdata = os.iorunv(program, argv, opt)
+    -- we need to get warning information from output
+    -- and we need to reserve warnings output from preprocessing
+    -- @see https://github.com/xmake-io/xmake/issues/5858
+    if outdata then
+        cppinfo.outdata = (cppinfo.outdata or "") .. outdata
+    end
+    if errdata then
+        cppinfo.errdata = (cppinfo.errdata or "") .. errdata
+    end
+end
+
+-- do compile
+function _compile(self, sourcefile, objectfile, compflags, opt)
+    opt = opt or {}
+    local program, argv = compargv(self, sourcefile, objectfile, compflags, opt)
+    local function _compile_fallback()
+        local runargv = argv
+        if is_host("windows") then
+            runargv = winos.cmdargv(argv, {escape = true})
+        end
+        return os.iorunv(program, runargv, {envs = self:runenvs(), shell = opt.shell})
+    end
+    local cppinfo
+    if distcc_build_client.is_distccjob() and distcc_build_client.singleton():has_freejobs() then
+        cppinfo = distcc_build_client.singleton():compile(program, argv, {envs = self:runenvs(),
+            preprocess = _preprocess, compile = _compile_preprocessed_file, compile_fallback = _compile_fallback,
+            tool = self, remote = true, shell = opt.shell})
+    elseif build_cache.is_enabled(opt.target) and build_cache.is_supported(self:kind()) then
+        cppinfo = build_cache.build(program, argv, {envs = self:runenvs(),
+            preprocess = _preprocess, compile = _compile_preprocessed_file, compile_fallback = _compile_fallback,
+            tool = self, shell = opt.shell})
+    end
+    if cppinfo then
+        return cppinfo.outdata, cppinfo.errdata
+    else
+        return _compile_fallback()
+    end
+end
+
+-- remove "-include xxx.h" and "-include-pch xxx.pch"
+function _remove_flags_for_pch(self, flags, opt)
+    opt = opt or {}
+    local result = {}
+    local include = false
+    local pchfile = opt.pchfile
+    for _, flag in ipairs(flags) do
+        local inserted = false
+        if not flag:startswith("-include") then
+            if not include then
+                inserted = true
+            end
+            include = false
+        else
+            include = true
+        end
+        if pchfile and flag:startswith("-fmodules") then
+            inserted = false
+        end
+        if inserted then
+            table.insert(result, flag)
+        end
+    end
+    return result
+end
+
+-- make the compile arguments list for the precompiled header
+function _translate_flags_for_pch(self, flags)
+    local pchflags = _remove_flags_for_pch(self, flags, {pchfile = true})
+    if self:kind() == "cxx" then
+        table.insert(pchflags, "-x")
+        table.insert(pchflags, "c++-header")
+    elseif self:kind() == "cc" then
+        table.insert(pchflags, "-x")
+        table.insert(pchflags, "c-header")
+    elseif self:kind() == "mxx" then
+        table.insert(pchflags, "-x")
+        table.insert(pchflags, "objective-c++-header")
+    elseif self:kind() == "mm" then
+        table.insert(pchflags, "-x")
+        table.insert(pchflags, "objective-c-header")
+    end
+    return pchflags
+end
+
+-- remove the force includes for c++modules
+-- @see https://github.com/xmake-io/xmake/issues/4051#issuecomment-2795707800
+function _translate_flags_for_mpp(self, flags)
+    return _remove_flags_for_pch(self, flags)
+end
+
+-- init it
 function init(self)
 
     -- init mxflags
@@ -189,19 +544,6 @@ function nf_vectorext(self, extension)
     return maps[extension]
 end
 
--- has -static-libstdc++?
-function _has_static_libstdcxx(self)
-    local has_static_libstdcxx = _g._HAS_STATIC_LIBSTDCXX
-    if has_static_libstdcxx == nil then
-        if self:has_flags("-static-libstdc++ -Werror", "ldflags", {flagskey = "gcc_static_libstdcxx"}) then
-            has_static_libstdcxx = true
-        end
-        has_static_libstdcxx = has_static_libstdcxx or false
-        _g._HAS_STATIC_LIBSTDCXX = has_static_libstdcxx
-    end
-    return has_static_libstdcxx
-end
-
 -- make the runtime flag
 function nf_runtime(self, runtime, opt)
     opt = opt or {}
@@ -358,20 +700,33 @@ function nf_linkgroup(self, linkgroup, opt)
     local flags = {}
     local extra = opt.extra
     if extra and not self:is_plat("macosx", "windows", "mingw") then
-        local group = extra.group
+        local as_needed = extra.as_needed
         local whole = extra.whole
-        if group and whole then
-            -- https://github.com/xmake-io/xmake/issues/4308
-            table.join2(flags, "-Wl,--whole-archive", "-Wl,--start-group", linkflags, "-Wl,--end-group", "-Wl,--no-whole-archive")
-        elseif group then
-            table.join2(flags, "-Wl,--start-group", linkflags, "-Wl,--end-group")
-        elseif whole then
-            table.join2(flags, "-Wl,--whole-archive", linkflags, "-Wl,--no-whole-archive")
-        end
+        local group = extra.group
         local static = extra.static
+        local prefix_flags = {}
+        local suffix_flags = {}
         if static then
-            table.join2(flags, "-Wl,-Bstatic", linkflags, "-Wl,-Bdynamic")
+            table.insert(prefix_flags, "-Wl,-Bstatic")
+            table.insert(suffix_flags, 1, "-Wl,-Bdynamic")
         end
+        -- https://github.com/xmake-io/xmake/issues/5621
+        if as_needed then
+            table.insert(prefix_flags, "-Wl,--as-needed")
+            table.insert(suffix_flags, 1, "-Wl,--no-as-needed")
+        elseif as_needed == false then
+            table.insert(prefix_flags, "-Wl,--no-as-needed")
+            table.insert(suffix_flags, 1, "-Wl,--as-needed")
+        end
+        if whole then
+            table.insert(prefix_flags, "-Wl,--whole-archive")
+            table.insert(suffix_flags, 1, "-Wl,--no-whole-archive")
+        end
+        if group then
+            table.insert(prefix_flags, "-Wl,--start-group")
+            table.insert(suffix_flags, 1, "-Wl,--end-group")
+        end
+        table.join2(flags, prefix_flags, linkflags, suffix_flags)
     end
     if #flags == 0 then
         flags = linkflags
@@ -386,7 +741,14 @@ end
 
 -- make the rpathdir flag
 function nf_rpathdir(self, dir, opt)
+    if self:is_plat("windows", "mingw") then
+        return
+    end
     opt = opt or {}
+    local extra = opt.extra
+    if extra and extra.installonly then
+        return
+    end
     dir = path.translate(dir)
     if self:has_flags("-Wl,-rpath=" .. dir, "ldflags") then
         local flags = {"-Wl,-rpath=" .. (dir:gsub("@[%w_]+", function (name)
@@ -395,7 +757,6 @@ function nf_rpathdir(self, dir, opt)
         end))}
         -- add_rpathdirs("...", {runpath = false})
         -- https://github.com/xmake-io/xmake/issues/5109
-        local extra = opt.extra
         if extra then
             if extra.runpath == false and self:has_flags("-Wl,-rpath=" .. dir .. ",--disable-new-dtags", "ldflags") then
                 flags[1] = flags[1] .. ",--disable-new-dtags"
@@ -478,7 +839,7 @@ function nf_pcheader(self, pcheaderfile, opt)
         if self:name() == "clang" then
             return {"-include", pcheaderfile, "-include-pch", pcoutputfile}
         else
-            return {"-include", path.filename(pcheaderfile), "-I", path.directory(pcoutputfile)}
+            return {"-I", path.directory(pcoutputfile), "-include", path.filename(pcheaderfile)}
         end
     end
 end
@@ -491,7 +852,7 @@ function nf_pcxxheader(self, pcheaderfile, opt)
         if self:name() == "clang" then
             return {"-include", pcheaderfile, "-include-pch", pcoutputfile}
         else
-            return {"-include", path.filename(pcheaderfile), "-I", path.directory(pcoutputfile)}
+            return {"-I", path.directory(pcoutputfile), "-include", path.filename(pcheaderfile)}
         end
     end
 end
@@ -504,7 +865,7 @@ function nf_pmheader(self, pcheaderfile, opt)
         if self:name() == "clang" then
             return {"-include", pcheaderfile, "-include-pch", pcoutputfile}
         else
-            return {"-include", path.filename(pcheaderfile), "-I", path.directory(pcoutputfile)}
+            return {"-I", path.directory(pcoutputfile), "-include", path.filename(pcheaderfile)}
         end
     end
 end
@@ -517,7 +878,7 @@ function nf_pmxxheader(self, pcheaderfile, opt)
         if self:name() == "clang" then
             return {"-include", pcheaderfile, "-include-pch", pcoutputfile}
         else
-            return {"-include", path.filename(pcheaderfile), "-I", path.directory(pcoutputfile)}
+            return {"-I", path.directory(pcoutputfile), "-include", path.filename(pcheaderfile)}
         end
     end
 end
@@ -554,8 +915,9 @@ function linkargv(self, objectfiles, targetkind, targetfile, flags, opt)
     end
 
     -- add `-Wl,--out-implib,outputdir/libxxx.a` for xxx.dll on mingw/gcc
-    if targetkind == "shared" and self:is_plat("mingw") then
-        table.insert(flags_extra, "-Wl,--out-implib," .. path.join(path.directory(targetfile), path.basename(targetfile) .. ".dll.a"))
+    local implibfile = _get_implibfile(self, targetkind, targetfile, opt)
+    if implibfile then
+        table.insert(flags_extra, "-Wl,--out-implib," .. implibfile)
     end
 
     -- init arguments
@@ -574,8 +936,14 @@ end
 --
 function link(self, objectfiles, targetkind, targetfile, flags, opt)
     opt = opt or {}
+
     os.mkdir(path.directory(targetfile))
-    local program, argv = linkargv(self, objectfiles, targetkind, targetfile, flags)
+    local implibfile = _get_implibfile(self, targetkind, targetfile, opt)
+    if implibfile then
+        os.mkdir(path.directory(implibfile))
+    end
+
+    local program, argv = linkargv(self, objectfiles, targetkind, targetfile, flags, opt)
     if option.get("verbose") then
         os.execv(program, argv, {envs = self:runenvs(), shell = opt.shell})
     else
@@ -583,279 +951,45 @@ function link(self, objectfiles, targetkind, targetfile, flags, opt)
     end
 end
 
--- has color diagnostics?
-function _has_color_diagnostics(self)
-    local colors_diagnostics = _g._HAS_COLOR_DIAGNOSTICS
-    if colors_diagnostics == nil then
-        if io.isatty() and (tty.has_color8() or tty.has_color256()) then
-            local theme = colors.theme()
-            if theme and theme:name() ~= "plain" then
-                -- for gcc
-                if self:has_flags("-fdiagnostics-color=always", "cxflags") then
-                    colors_diagnostics = "-fdiagnostics-color=always"
-                -- for clang
-                elseif self:has_flags("-fcolor-diagnostics", "cxflags") then
-                    colors_diagnostics = "-fcolor-diagnostics"
-                end
-
-                -- enable color output for windows, @see https://github.com/xmake-io/xmake-vscode/discussions/260
-                if colors_diagnostics and self:name() == "clang" and is_host("windows") and
-                    self:has_flags("-fansi-escape-codes", "cxflags") then
-                    colors_diagnostics = table.join(colors_diagnostics, "-fansi-escape-codes")
-                end
-            end
-        end
-        colors_diagnostics = colors_diagnostics or false
-        _g._HAS_COLOR_DIAGNOSTICS = colors_diagnostics
-    end
-    return colors_diagnostics
-end
-
--- get preprocess file path
-function _get_cppfile(sourcefile, objectfile)
-    return path.join(path.directory(objectfile), "__cpp_" .. path.basename(objectfile) .. path.extension(sourcefile))
-end
-
--- do preprocess
-function _preprocess(program, argv, opt)
-
-    -- is gcc or clang?
-    local tool = opt.tool
-    local is_gcc = false
-    local is_clang = false
-    if tool then
-        if tool:name() == "gcc" or tool:name() == "gxx" then
-            is_gcc = true
-        elseif tool:name():startswith("clang") then
-            is_clang = true
-        elseif tool:name() == "circle" then
-            return
-        end
-    end
-
-    -- enable "-fdirectives-only"? we need to enable it manually
-    --
-    -- @see https://github.com/xmake-io/xmake/issues/2603
-    -- https://github.com/xmake-io/xmake/issues/2425
-    local directives_only
-    if is_gcc then
-        local cachekey = "core.tools." .. tool:name()
-        directives_only = memcache.get(cachekey, "directives_only")
-        if directives_only == nil then
-            if os.isfile(os.projectfile()) and project.policy("preprocessor.gcc.directives_only") then
-                directives_only = true
-            end
-            memcache.set(cachekey, "directives_only", directives_only)
-        end
-    end
-
-    -- get flags and source file
-    local flags = {}
-    local cppflags = {}
-    local skipped = program:endswith("cache") and 1 or 0
-    for _, flag in ipairs(argv) do
-        if flag == "-o" then
-            break
-        end
-
-        -- get preprocessor flags
-        table.insert(cppflags, flag)
-
-        -- for c++ modules, we cannot support it for clang now
-        if is_clang and flag:startswith("-fmodules") then
-            return
-        end
-
-        -- we cannot enable "-fdirectives-only"
-        if directives_only and (flag:startswith("-D__TIME__=") or
-                flag:startswith("-D__DATE__=") or flag:startswith("-D__TIMESTAMP__=")) then
-            directives_only = false
-        end
-
-        -- get compiler flags
-        if flag == "-MMD" or (flag:startswith("-I") and #flag > 2) or flag:startswith("--sysroot=") then
-            skipped = 1
-        elseif flag == "-MF" or
-            flag == "-I" or flag == "-isystem" or flag == "-include" or flag == "-include-pch" or
-            flag == "-isysroot" or flag == "-gcc-toolchain" then
-            skipped = 2
-        elseif flag:endswith("xcrun") then
-            skipped = 4
-        end
-        if skipped > 0 then
-            skipped = skipped - 1
-        else
-            table.insert(flags, flag)
-        end
-    end
-    local objectfile = argv[#argv - 1]
-    local sourcefile = argv[#argv]
-    assert(objectfile and sourcefile, "%s: iorunv(%s): invalid arguments!", self, program)
-
-    -- is precompiled header?
-    if objectfile:endswith(".gch") or objectfile:endswith(".pch") then
-        return
-    end
-
-    -- disable linemarkers?
-    local linemarkers = _g.linemarkers
-    if linemarkers == nil then
-        if os.isfile(os.projectfile()) and project.policy("preprocessor.linemarkers") == false then
-            linemarkers = false
-        else
-            linemarkers = true
-        end
-        _g.linemarkers = linemarkers
-    end
-
-    -- do preprocess
-    local cppfile = _get_cppfile(sourcefile, objectfile)
-    local cppfiledir = path.directory(cppfile)
-    if not os.isdir(cppfiledir) then
-        os.mkdir(cppfiledir)
-    end
-    table.insert(cppflags, "-E")
-    -- it will be faster for preprocessing
-    -- when preprocessing, handle directives, but do not expand macros.
-    if directives_only then
-        table.insert(cppflags, "-fdirectives-only")
-    end
-    if linemarkers == false then
-        table.insert(cppflags, "-P")
-    end
-    table.insert(cppflags, "-o")
-    table.insert(cppflags, cppfile)
-    table.insert(cppflags, sourcefile)
-
-    -- we need to mark as it when compiling the preprocessed source file
-    -- it will indicate to the preprocessor that the input file has already been preprocessed.
-    if is_gcc then
-        table.insert(flags, "-fpreprocessed")
-    end
-    -- with -fpreprocessed, predefinition of command line and most builtin macros is disabled.
-    if directives_only then
-        table.insert(flags, "-fdirectives-only")
-    end
-
-    -- do preprocess
-    local cppinfo = try {function ()
-        local outdata, errdata = os.iorunv(program, cppflags, opt)
-        return {outdata = outdata, errdata = errdata,
-                sourcefile = sourcefile, objectfile = objectfile, cppfile = cppfile, cppflags = flags}
-    end}
-    if not cppinfo then
-        if is_gcc then
-            local cachekey = "core.tools." .. tool:name()
-            memcache.set(cachekey, "directives_only", false)
-        end
-    end
-    return cppinfo
-end
-
--- compile preprocessed file
-function _compile_preprocessed_file(program, cppinfo, opt)
-    local outdata, errdata = os.iorunv(program, table.join(cppinfo.cppflags, "-o", cppinfo.objectfile, cppinfo.cppfile), opt)
-    -- we need to get warning information from output
-    cppinfo.outdata = outdata
-    cppinfo.errdata = errdata
-end
-
--- do compile
-function _compile(self, sourcefile, objectfile, compflags, opt)
-    opt = opt or {}
-    local program, argv = compargv(self, sourcefile, objectfile, compflags)
-    local function _compile_fallback()
-        return os.iorunv(program, argv, {envs = self:runenvs(), shell = opt.shell})
-    end
-    local cppinfo
-    if distcc_build_client.is_distccjob() and distcc_build_client.singleton():has_freejobs() then
-        cppinfo = distcc_build_client.singleton():compile(program, argv, {envs = self:runenvs(),
-            preprocess = _preprocess, compile = _compile_preprocessed_file, compile_fallback = _compile_fallback,
-            tool = self, remote = true, shell = opt.shell})
-    elseif build_cache.is_enabled(opt.target) and build_cache.is_supported(self:kind()) then
-        cppinfo = build_cache.build(program, argv, {envs = self:runenvs(),
-            preprocess = _preprocess, compile = _compile_preprocessed_file, compile_fallback = _compile_fallback,
-            tool = self, shell = opt.shell})
-    end
-    if cppinfo then
-        return cppinfo.outdata, cppinfo.errdata
-    else
-        return _compile_fallback()
-    end
-end
-
--- make the compile arguments list for the precompiled header
-function _compargv_pch(self, pcheaderfile, pcoutputfile, flags)
-
-    -- remove "-include xxx.h" and "-include-pch xxx.pch"
-    local pchflags = {}
-    local include = false
-    for _, flag in ipairs(flags) do
-        if not flag:find("-include", 1, true) then
-            if not include then
-                table.insert(pchflags, flag)
-            end
-            include = false
-        else
-            include = true
-        end
-    end
-
-    -- set the language of precompiled header?
-    if self:kind() == "cxx" then
-        table.insert(pchflags, "-x")
-        table.insert(pchflags, "c++-header")
-    elseif self:kind() == "cc" then
-        table.insert(pchflags, "-x")
-        table.insert(pchflags, "c-header")
-    elseif self:kind() == "mxx" then
-        table.insert(pchflags, "-x")
-        table.insert(pchflags, "objective-c++-header")
-    elseif self:kind() == "mm" then
-        table.insert(pchflags, "-x")
-        table.insert(pchflags, "objective-c-header")
-    end
-
-    -- make the compile arguments list
-    return self:program(), table.join("-c", pchflags, "-o", pcoutputfile, pcheaderfile)
-end
-
 -- make the compile arguments list
-function compargv(self, sourcefile, objectfile, flags)
-    -- precompiled header?
+function compargv(self, sourcefile, objectfile, flags, opt)
+
+    -- is precompiled header or module files? remove the force includes.
     local extension = path.extension(sourcefile)
     if (extension:startswith(".h") or extension == ".inl") then
-        return _compargv_pch(self, sourcefile, objectfile, flags)
+        flags = _translate_flags_for_pch(self, flags)
+    elseif support.has_module_extension(sourcefile, {extension = extension}) then
+        flags = _translate_flags_for_mpp(self, flags)
     end
-    return self:program(), table.join("-c", flags, "-o", objectfile, sourcefile)
+
+    local argv = table.join("-c", flags, "-o", objectfile, sourcefile)
+    return self:program(), argv
 end
 
 -- compile the source file
 function compile(self, sourcefile, objectfile, dependinfo, flags, opt)
-
-    -- ensure the object directory
+    opt = opt or {}
     os.mkdir(path.directory(objectfile))
 
-    -- compile it
-    opt = opt or {}
     local depfile = dependinfo and os.tmpfile() or nil
     try
     {
         function ()
 
-            -- support `-MMD -MF depfile.d`? some old gcc does not support it at same time
-            if depfile and _g._HAS_MMD_MF == nil then
-                _g._HAS_MMD_MF = self:has_flags({"-MMD", "-MF", os.nuldev()}, "cxflags", { flagskey = "-MMD -MF" }) or false
-            end
-
             -- generate includes file
             local compflags = flags
-            if depfile and _g._HAS_MMD_MF then
-                compflags = table.join(compflags, "-MMD", "-MF", depfile)
+            if depfile then
+                if _is_cosmocc(self) then
+                    depfile = path.unix(depfile)
+                end
+                local depfile_flags = _get_depfile_flags(self)
+                if depfile_flags then
+                    compflags = table.join(compflags, depfile_flags, depfile)
+                end
             end
 
-            -- has color diagnostics? enable it
-            local colors_diagnostics = _has_color_diagnostics(self)
+            -- attempt to enable color diagnostics
+            local colors_diagnostics = _get_color_diagnostics_flag(self)
             if colors_diagnostics then
                 compflags = table.join(compflags, colors_diagnostics)
             end
@@ -922,7 +1056,8 @@ function compile(self, sourcefile, objectfile, dependinfo, flags, opt)
                 -- generate the dependent includes
                 if depfile and os.isfile(depfile) then
                     if dependinfo then
-                        dependinfo.depfiles_gcc = io.readfile(depfile, {continuation = "\\"})
+                        dependinfo.depfiles_format = "gcc"
+                        dependinfo.depfiles = io.readfile(depfile, {continuation = "\\"})
                     end
 
                     -- remove the temporary dependent file
